﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Interop;
using System.Linq;
using System.IO;

namespace STDHelper
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private DispatcherTimer t;
        private BitmapSource lastImage;
        public MainWindow()
        {
            InitializeComponent();
            System.Windows.Controls.ComboBox cb = FindName("cb_Screens") as System.Windows.Controls.ComboBox;
            Screen[] screens = Screen.AllScreens;
            foreach(Screen s in screens)
            {
                cb.Items.Add(s.DeviceName);
            }
            cb.SelectedItem = cb.Items.GetItemAt(0);

            t = new DispatcherTimer();
            // Hook up the Elapsed event for the timer. 
            t.Tick += new EventHandler(OnTimedEvent);
            t.Interval = new TimeSpan(0, 0, 1);
        }

        private WriteableBitmap getAreaFromBitmap(BitmapSource src, Int32Rect area)
        {
            // Calculate stride of source
            int stride = src.PixelWidth * (src.Format.BitsPerPixel + 7) / 8;

            // Create data array to hold source pixel data
            byte[] data = new byte[stride * area.Height];

            // Copy source image pixels to the data array
            src.CopyPixels(area, data, stride, 0);

            // Create WriteableBitmap to copy the pixel data to.      
            WriteableBitmap target = new WriteableBitmap(
              area.Width,
              area.Height,
              src.DpiX, src.DpiY,
              src.Format, null);

            // Write the pixel data to the WriteableBitmap.
            target.WritePixels(
              new Int32Rect(0, 0, area.Width, area.Height),
              data, stride, 0);

            // Set the WriteableBitmap as the source for the <Image> element 
            // in XAML so you can see the result of the copy
            return target;
        }

        private void Btn_Switch_Click(object sender, RoutedEventArgs e)
        {
            System.Windows.Controls.Image i1 = FindName("image1") as System.Windows.Controls.Image;
            System.Windows.Controls.Image i2 = FindName("image2") as System.Windows.Controls.Image;
            if (t.IsEnabled)
            {
                t.Stop();
                i1.Visibility = Visibility.Visible;
                i1.Opacity = 0.5;
                i2.Visibility = Visibility.Visible;
                i2.Opacity = 0.5;
            }
            else
            {
                i1.Opacity = 1;
                i2.Opacity = 1;
                t.Start();
            }
        }

        private void OnTimedEvent(object sender, EventArgs e)
        {
            Debug.WriteLine("TICK");
            System.Windows.Controls.Image i1 = FindName("image1") as System.Windows.Controls.Image;
            System.Windows.Controls.Image i2 = FindName("image2") as System.Windows.Controls.Image;
            Debug.WriteLine("i1 " + i1.Visibility);
            Debug.WriteLine("i2 " + i2.Visibility);
            if (i1.Visibility == Visibility.Hidden)
            {
                i1.Visibility = Visibility.Visible;
                i2.Visibility = Visibility.Hidden;
            } else
            {
                i1.Visibility = Visibility.Hidden;
                i2.Visibility = Visibility.Visible;
            }
        }

        private void Btn_LoadScreenshot_Click(object sender, RoutedEventArgs e)
        {
            System.Windows.Controls.Image i1 = FindName("image1") as System.Windows.Controls.Image;
            System.Windows.Controls.Image i2 = FindName("image2") as System.Windows.Controls.Image;
            System.Windows.Controls.ComboBox cb = FindName("cb_Screens") as System.Windows.Controls.ComboBox;
            string scrStr = cb.SelectedItem as string;
            Screen screen = Screen.AllScreens.Where(elem => elem.DeviceName == scrStr).SingleOrDefault();
            BitmapSource bms = CopyScreen(screen);
            WriteableBitmap crop1 = getAreaFromBitmap(bms, new Int32Rect(226, 123, 450, 450));
            WriteableBitmap crop2 = getAreaFromBitmap(bms, new Int32Rect(685, 123, 450, 450));
            i1.Source = crop1;
            i1.Opacity = 0.5;
            i2.Source = crop2;
            i2.Opacity = 0.5;
        }

        private static BitmapSource CopyScreen(Screen screen)
        {
            var left = screen.Bounds.X;
            var top = screen.Bounds.Y;
            var right = screen.Bounds.X + screen.Bounds.Width;
            var bottom = screen.Bounds.Y + screen.Bounds.Height;
            var width = right - left;
            var height = bottom - top;

            using (var screenBmp = new Bitmap(width, height, System.Drawing.Imaging.PixelFormat.Format32bppArgb))
            {
                using (var bmpGraphics = Graphics.FromImage(screenBmp))
                {
                    bmpGraphics.CopyFromScreen(left, top, 0, 0, new System.Drawing.Size(width, height));
                    return Imaging.CreateBitmapSourceFromHBitmap(
                        screenBmp.GetHbitmap(),
                        IntPtr.Zero,
                        Int32Rect.Empty,
                        BitmapSizeOptions.FromEmptyOptions());
                }
            }
        }

        private void Btn_DbgSave_Click(object sender, RoutedEventArgs e)
        {
            Debug.WriteLine("SAVE CLICK");
            using (var fileStream = new FileStream("debug.png", FileMode.Create))
            {
                Debug.WriteLine("START SAVE");
                BitmapEncoder encoder = new PngBitmapEncoder();
                encoder.Frames.Add(BitmapFrame.Create(this.lastImage));
                encoder.Save(fileStream);
            }
        }
    }

    public class DraggableExtender : DependencyObject
    {
        // This is the dependency property we're exposing - we'll 
        // access this as DraggableExtender.CanDrag="true"/"false"
        public static readonly DependencyProperty CanDragProperty =
            DependencyProperty.RegisterAttached("CanDrag",
            typeof(bool),
            typeof(DraggableExtender),
            new UIPropertyMetadata(false, OnChangeCanDragProperty));

        // The expected static setter
        public static void SetCanDrag(UIElement element, bool o)
        {
            element.SetValue(CanDragProperty, o);
        }

        // the expected static getter
        public static bool GetCanDrag(UIElement element)
        {
            return (bool)element.GetValue(CanDragProperty);
        }

        // This is triggered when the CanDrag property is set. We'll
        // simply check the element is a UI element and that it is
        // within a canvas. If it is, we'll hook into the mouse events
        private static void OnChangeCanDragProperty(DependencyObject d,
                  DependencyPropertyChangedEventArgs e)
        {
            UIElement element = d as UIElement;
            if (element == null) return;

            if (e.NewValue != e.OldValue)
            {
                if ((bool)e.NewValue)
                {
                    element.PreviewMouseDown += element_PreviewMouseDown;
                    element.PreviewMouseUp += element_PreviewMouseUp;
                    element.PreviewMouseMove += element_PreviewMouseMove;
                }
                else
                {
                    element.PreviewMouseDown -= element_PreviewMouseDown;
                    element.PreviewMouseUp -= element_PreviewMouseUp;
                    element.PreviewMouseMove -= element_PreviewMouseMove;
                }
            }
        }

        // Determine if we're presently dragging
        private static bool _isDragging = false;
        // The offset from the top, left of the item being dragged 
        // and the original mouse down
        private static System.Windows.Point _offset;

        // This is triggered when the mouse button is pressed 
        // on the element being hooked
        static void element_PreviewMouseDown(object sender,
                System.Windows.Input.MouseButtonEventArgs e)
        {
            
            // Ensure it's a framework element as we'll need to 
            // get access to the visual tree
            FrameworkElement element = sender as FrameworkElement;
            Debug.WriteLine("MOUSEDOWN "+element.Name+" drg is "+_isDragging);
            if (element == null) return;

            // start dragging and get the offset of the mouse 
            // relative to the element
            _isDragging = true;
            _offset = e.GetPosition(element);
        }

        // This is triggered when the mouse is moved over the element
        private static void element_PreviewMouseMove(object sender,
                  System.Windows.Input.MouseEventArgs e)
        {
            FrameworkElement element = sender as FrameworkElement;
            Debug.WriteLine("MMOVE " + element.Name + " drg is " + _isDragging);
            // If we're not dragging, don't bother - also validate the element
            if (!_isDragging) return;

            Debug.WriteLine("DRAGGING " + element.Name + " drg is " + _isDragging);

            if (element == null) return;

            var canvas = element.Parent as Canvas;
            if (canvas == null) return;

            // Get the position of the mouse relative to the canvas
            System.Windows.Point mousePoint = e.GetPosition(canvas);

            // Offset the mouse position by the original offset position
            mousePoint.Offset(-_offset.X, -_offset.Y);

            // Move the element on the canvas
            element.SetValue(Canvas.LeftProperty, mousePoint.X);
            element.SetValue(Canvas.TopProperty, mousePoint.Y);
        }

        // this is triggered when the mouse is released
        private static void element_PreviewMouseUp(object sender,
                MouseButtonEventArgs e)
        {
            _isDragging = false;
            _offset = new System.Windows.Point();
        }

    }
}
